/* ------------------------------------------------ */
/*                      PUBLIC                      */
/* ------------------------------------------------ */

// Create the Flickercode by specification HHD 1.0.1
function hhd101Flicker(startCodeValue, bde1Value, bde2Value) {
	var startCode;		//Startcode
	var startLength; 	//LS
	var bde1Code;		//BDE1 
	var bde1Length;
	var bde2Code;		//BDE2
	var bbe2Length;
	var length;			//LC
	var bde1Typ, bde2Typ;

	// Startcode
	if (startCodeValue == "") {
		console.error("Please insert a Startcode!");
		return false;
	} else {
		if (containsNonDigits(startCodeValue) == true) {				// ASCII values
			console.error("Please insert only numbers in the Startcode!");
			return false;
		} else if (containsNonDigits(startCodeValue) == false) {	// Non ASCII values
			startCode = normalizeNonASCIIElements(startCodeValue);
			startLength = startCode.length /= 2;
		}
	}

	// BDE 1
	if (containsNonDigits(bde1Value) == true) {						// ASCII values		
		bde1Length = bde1Value.length;
		bde1Code = getASCIIHexFormatForZKACharacterSet(bde1Value);
		bde1Typ = 1;
	} else if (containsNonDigits(bde1Value) == false) {				// Non ASCII values
		bde1Code = normalizeNonASCIIElements(bde1Value);
		bde1Length = bde1Code.length / 2;
		bde1Typ = 0;
	}

	// BDE 2
	if (containsNonDigits(bde2Value) == true) {						// ASCII values		
		bbe2Length = bde2Value.length;
		bde2Code = getASCIIHexFormatForZKACharacterSet(bde2Value);
		bde2Typ = 1;
	} else if (containsNonDigits(bde2Value) == false) {				// Non ASCII values
		bde2Code = normalizeNonASCIIElements(bde2Value);
		bbe2Length = bde2Code.length / 2;
		bde2Typ = 0;
	}

	length = 1 + 1 + startLength;									// LC=LS+CB+start_len
	if (containsNonDigits(startCodeValue) == true) {				// Startcode is an ASCII value
		startLength += 0x10;
	}

	// Luhn, Xor, Flickercode				
	var luhn = luhnCalc(startCode + bde1Code + bde2Code);			// calculate luhn
	var xorData = getXorDataV101(startLength, startCode, bde1Length, bde1Code, bbe2Length, bde2Code, length, bde1Typ, bde2Typ);
	var xor = xorCalc(xorData);										// calculate xor		
	var flickerCode = xorData + sprintf("%X%X", luhn, xor);			// Flickercode: LC; LS; Start-Code; LBDE1; BDE1; LBDE2; BDE2; set Luhn; set xor
	return flickerCode;
}

// Create the Flickercode by specification HHD 1.4
function hhd14Flicker(startCodeValue, bde1Value, bde2Value, bde3Value) {
	var startCode;		//Start-Code
	var startLength; 	//LS
	var bde1Code;		//BDE1
	var bde1Length;
	var bde2Code;		//BDE2
	var bde2Length;
	var bde3Code;		//BDE3
	var bde3Length;
	var length;			//LC
	var startCodeTyp, bde1Typ, bde2Typ, bde3Typ;

	// Startcode
	if (startCodeValue == "") {
		console.error("Please insert a Startcode!");
		return false;
	} else {
		if (containsNonDigits(startCodeValue) == true) {				// ASCII values
			console.error("Please insert only numbers in the Startcode!");
			return false;
		} else if (containsNonDigits(startCodeValue) == false) {	// Non ASCII values
			startCode = normalizeNonASCIIElements(startCodeValue);
			startLength = startCode.length /= 2;
			startCodeTyp = 0;
		}
	}

	// BDE 1
	if (containsNonDigits(bde1Value) == true) {						// ASCII values		
		bde1Length = bde1Value.length;
		bde1Code = getASCIIHexFormatForZKACharacterSet(bde1Value);
		bde1Typ = 1;
	} else if (containsNonDigits(bde1Value) == false) {				// Non ASCII values
		bde1Code = normalizeNonASCIIElements(bde1Value);
		bde1Length = bde1Code.length / 2;
		bde1Typ = 0;
	}

	// BDE 2
	if (containsNonDigits(bde2Value) == true) {						// ASCII values		
		bde2Length = bde2Value.length;
		bde2Code = getASCIIHexFormatForZKACharacterSet(bde2Value);
		bde2Typ = 1;
	} else if (containsNonDigits(bde2Value) == false) {				// Non ASCII values
		bde2Code = normalizeNonASCIIElements(bde2Value);
		bde2Length = bde2Code.length / 2;
		bde2Typ = 0;
	}

	// BDE 3
	if (containsNonDigits(bde3Value) == true) {						// ASCII values		
		bde3Length = bde3Value.length;
		bde3Code = getASCIIHexFormatForZKACharacterSet(bde3Value);
		bde3Typ = 1;
	} else if (containsNonDigits(bde3Value) == false) {				// Non ASCII values
		bde3Code = normalizeNonASCIIElements(bde3Value);
		bde3Length = bde3Code.length / 2;
		bde3Typ = 0;
	}

	length = 1 + 1 + 1 + startLength;													// LC=LS+CB+controlByte+start_len
	controlByte = 0x1;																	// xor_data: set controlByte (HHD 1.4)
	controlByte = "0" + controlByte;													// xor_data: set controlByte (no other Control Byte)															

	// Luhn, Xor, Flickercode				
	var luhn = luhnCalc(controlByte + startCode + bde1Code + bde2Code + bde3Code);		// calculate luhn
	var xorData = getXorDataV14(startLength, startCode, bde1Length, bde1Code, bde2Length, bde2Code, length, bde3Length, bde3Code, startCodeTyp, bde1Typ, bde2Typ, bde3Typ);
	var xor = xorCalc(xorData);															// calculate xor		
	var flickerCode = xorData + sprintf("%X%X", luhn, xor);								// Flickercode: LC; LS; Start-Code; LBDE1; BDE1; LBDE2; BDE2; LBDE3; BDE3; set Luhn; set xor
	return flickerCode;
}

// Create the Flicker-Code by specification Stuza (HHD 1.4)
function stuzzaHHD14Flicker(startCodeValue, bde1Value, bde2Value, bde3Value, bde4Value, bde5Value, bde6Value) {
	var startCode;		//Start-Code
	var startLength; 	//LS
	var bde1Code;		//BDE1
	var bde1Length;
	var bde2Code;		//BDE2
	var bde2Length;
	var bde3Code;		//BDE3
	var bde3Length;
	var bde4Code;		//BDE4
	var bde4Length;
	var bde5Code;		//BDE5
	var bde5Length;
	var bde6Code;		//BDE6
	var bde6Length;
	var length;			//LC
	var startCodeTyp, bde1Typ, bde2Typ, bde3Typ, bde4Typ, bde5Typ, bde6Typ;

	// Startcode
	if (startCodeValue == "") {
		console.error("Please insert a Startcode!");
		return false;
	} else {
		if (containsNonDigits(startCodeValue) == true) {				// ASCII values
			console.error("Please insert only numbers in the Startcode!");
			return false;
		} else if (containsNonDigits(startCodeValue) == false) {	// Non ASCII values
			startCode = normalizeNonASCIIElements(startCodeValue);
			startLength = startCode.length /= 2;
			startCodeTyp = 0;
		}
	}

	// BDE 1
	if (containsNonDigits(bde1Value) == true) {						// ASCII values		
		bde1Length = bde1Value.length;
		bde1Code = getASCIIHexFormatForZKACharacterSet(bde1Value);
		bde1Typ = 1;
	} else if (containsNonDigits(bde1Value) == false) {				// Non ASCII values
		bde1Code = normalizeNonASCIIElements(bde1Value);
		bde1Length = bde1Code.length / 2;
		bde1Typ = 0;
	}

	// BDE 2
	if (containsNonDigits(bde2Value) == true) {						// ASCII values		
		bde2Length = bde2Value.length;
		bde2Code = getASCIIHexFormatForZKACharacterSet(bde2Value);
		bde2Typ = 1;
	} else if (containsNonDigits(bde2Value) == false) {				// Non ASCII values
		bde2Code = normalizeNonASCIIElements(bde2Value);
		bde2Length = bde2Code.length / 2;
		bde2Typ = 0;
	}

	// BDE 3
	if (containsNonDigits(bde3Value) == true) {						// ASCII values		
		bde3Length = bde3Value.length;
		bde3Code = getASCIIHexFormatForZKACharacterSet(bde3Value);
		bde3Typ = 1;
	} else if (containsNonDigits(bde3Value) == false) {				// Non ASCII values
		bde3Code = normalizeNonASCIIElements(bde3Value);
		bde3Length = bde3Code.length / 2;
		bde3Typ = 0;
	}

	// BDE 4
	if (containsNonDigits(bde4Value) == true) {						// ASCII values		
		bde4Length = bde4Value.length;
		bde4Code = getASCIIHexFormatForZKACharacterSet(bde4Value);
		bde4Typ = 1;
	} else if (containsNonDigits(bde4Value) == false) {				// Non ASCII values
		bde4Code = normalizeNonASCIIElements(bde4Value);
		bde4Length = bde4Code.length / 2;
		bde4Typ = 0;
	}

	// BDE 5
	if (containsNonDigits(bde5Value) == true) {						// ASCII values		
		bde5Length = bde5Value.length;
		bde5Code = getASCIIHexFormatForZKACharacterSet(bde5Value);
		bde5Typ = 1;
	} else if (containsNonDigits(bde5Value) == false) {				// Non ASCII values
		bde5Code = normalizeNonASCIIElements(bde5Value);
		bde5Length = bde5Code.length / 2;
		bde5Typ = 0;
	}

	// BDE 6
	if (containsNonDigits(bde6Value) == true) {						// ASCII values		
		bde6Length = bde6Value.length;
		bde6Code = getASCIIHexFormatForZKACharacterSet(bde6Value);
		bde6Typ = 1;
	} else if (containsNonDigits(bde6Value) == false) {				// Non ASCII values
		bde6Code = normalizeNonASCIIElements(bde6Value);
		bde6Length = bde6Code.length / 2;
		bde6Typ = 0;
	}

	length = 1 + 1 + 1 + 1 + startLength;												// LC=LS+CB+two controlBytes+start_len
	controlByte = 0x0;																	// xor_data: set controlByte (HHD 1.4)
	controlByte = "8" + controlByte;													// xor_data: set controlByte (other Control Byte follow)															
	secondControlByte = "0" + "9";														// ControlByte for Austria, Version 0001, no other controlByte

	// Luhn, Xor, Flickercode				
	var luhn = luhnCalc(controlByte + secondControlByte + startCode + bde1Code + bde2Code + bde3Code + bde4Code + bde5Code + bde6Code);	// calculate luhn
	var xorData = getXorDataV14Stuzza(startLength, startCode, bde1Length, bde1Code, bde2Length, bde2Code, length, bde3Length, bde3Code, bde4Length, bde4Code, bde5Length, bde5Code, bde6Length, bde6Code, startCodeTyp, bde1Typ, bde2Typ, bde3Typ, bde4Typ, bde5Typ, bde6Typ);
	var xor = xorCalc(xorData);															// calculate xor		
	var flickerCode = xorData + sprintf("%X%X", luhn, xor);								// Flickercode: LC; LS; Start-Code; LBDE1; BDE1; LBDE2; BDE2; LBDE3; BDE3; LBDE4; BDE4; LBDE5; BDE5; LBDE6; BDE6; set Luhn; set xor
	return flickerCode;
}

//Create the Flicker-Code by specification SecOPTIC
function secOPTICFlicker(startCodeValue, bde1Value, bde2Value) {
	var startCode;		//Start-Code
	var startLength; 	//LS
	var bde1Code;		//BDE1
	var bde1Length;
	var bde2Code;		//BDE2
	var bde2Length;
	var length;			//LC
	var startCodeTyp, bde1Typ, bde2Typ;

	// Startcode
	if (startCodeValue == "") {
		console.error("Please insert a Startcode!");
		return false;
	} else {
		if (containsNonDigits(startCodeValue) == true) {				// ASCII values
			console.error("Please insert only numbers in the Startcode!");
			return false;
		} else if (containsNonDigits(startCodeValue) == false) {	// Non ASCII values
			startCode = normalizeNonASCIIElements(startCodeValue);
			startLength = startCode.length /= 2;
			startCodeTyp = 0;
		}
	}

	// BDE 1
	if (containsNonDigits(bde1Value) == true) {						// ASCII values		
		bde1Length = bde1Value.length;
		bde1Code = getASCIIHexFormatForSecOPTICCharacterSet(bde1Value);
		bde1Typ = 1;
	} else if (containsNonDigits(bde1Value) == false) {				// Non ASCII values
		bde1Code = normalizeNonASCIIElements(bde1Value);
		bde1Length = bde1Code.length / 2;
		bde1Typ = 0;
	}

	// BDE 2
	if (containsNonDigits(bde2Value) == true) {						// ASCII values		
		bde2Length = bde2Value.length;
		bde2Code = getASCIIHexFormatForSecOPTICCharacterSet(bde2Value);
		bde2Typ = 1;
	} else if (containsNonDigits(bde2Value) == false) {				// Non ASCII values
		bde2Code = normalizeNonASCIIElements(bde2Value);
		bde2Length = bde2Code.length / 2;
		bde2Typ = 0;
	}

	length = 1 + 1 + 1 + startLength;													// LC=LS+CB+controlByte+start_len
	controlByte = 0x1;																	// xor_data: set controlByte (HHD 1.4)
	controlByte = "0" + controlByte;													// xor_data: set controlByte (no other Control Byte)															

	// Luhn, Xor, Flickercode				
	var luhn = luhnCalc(controlByte + startCode + bde1Code + bde2Code);					// calculate luhn
	var xorData = getXorDataSecOPTIC(startLength, startCode, bde1Length, bde1Code, bde2Length, bde2Code, length, startCodeTyp, bde1Typ, bde2Typ);
	var xor = xorCalc(xorData);															// calculate xor		
	var flickerCode = xorData + sprintf("%X%X", luhn, xor);								// Flickercode: LC; LS; Start-Code; LBDE1; BDE1; LBDE2; BDE2; LBDE3; BDE3; set Luhn; set xor
	return flickerCode;
}

/* ------------------------------------------------ */
/*                      PRIVATE                     */
/* ------------------------------------------------ */

// Non ASCII elements (For example: A BCD element) become normalize.
function normalizeNonASCIIElements() {
	var result = arguments[0];

	if ((result.length % 2) == 1) {
		result = result + 'F';
	}
	return result;
}

// Check if the comittet parameter contains non digits
function containsNonDigits() {
	var cut = arguments[0];
	var i;

	for (i = 0; i < cut.length; ++i) {
		if (cut.charAt(i) < "0" || cut.charAt(i) > "9") {
			return true;
		}
	}
	return false;
}

// Change the committed parameter to ASCIIHexFormat
function getASCIIHexFormatForZKACharacterSet(elements) {
	var acharCode;
	var asciiElements = "";

	for (var i = 0; i < elements.length; ++i) {
		acharCode = elements.charCodeAt(i);

		if (acharCode == 8364) { 	// Euro-symbol
			acharCode = 36;
		}
		if (acharCode == 196) {		// �	
			acharCode = 91;
		}
		if (acharCode == 214) {		// �
			acharCode = 92;
		}
		if (acharCode == 220) {		// �	
			acharCode = 93;
		}
		if (acharCode == 163) {		// Pound-symbol
			acharCode = 94;
		}
		if (acharCode == 228) {		// �
			acharCode = 123;
		}
		if (acharCode == 246) {		// �
			acharCode = 124;
		}
		if (acharCode == 252) {		// �
			acharCode = 125;
		}
		if (acharCode == 223) {		// �
			acharCode = 126;
		}

		if (acharCode > 0xff) {
			console.error("Error, wrong char");
		}

		asciiElements += sprintf("%02X", acharCode);
	}
	return asciiElements;
}

// Change the committed parameter to ASCIIHexFormat
function getASCIIHexFormatForSecOPTICCharacterSet(elements) {
	var acharCode;
	var asciiElements = "";

	for (var i = 0; i < elements.length; ++i) {
		acharCode = elements.charCodeAt(i);

		if (acharCode == 8364) { 	// Euro-symbol
			acharCode = 164;
		}
		if (acharCode == 286) {
			acharCode = 208;
		}
		if (acharCode == 287) {
			acharCode = 241;
		}
		if (acharCode == 304) {
			acharCode = 221;
		}
		if (acharCode == 305) {
			acharCode = 253;
		}
		if (acharCode == 350) {
			acharCode = 222;
		}
		if (acharCode == 351) {
			acharCode = 254;
		}

		if (acharCode > 0xff) {
			console.error("Error, wrong char");
		}

		asciiElements += sprintf("%02X", acharCode);
	}
	return asciiElements;
}

function sprintf() {
	var i = 0, a, f = arguments[i++], o = [], m, p, c, x;
	while (f) {
		if (m = /^[^\x25]+/.exec(f)) {
			o.push(m[0]);
		} else if (m = /^\x25{2}/.exec(f)) {
			o.push('%');
		} else if (m = /^\x25(?:(\d+)\$)?(\+)?(0|'[^$])?(-)?(\d+)?(?:\.(\d+))?([b-fosuxX])/.exec(f)) {
			if (((a = arguments[m[1] || i++]) == null) || (a == undefined)) {
				throw ("Too few arguments.");
			}
			if (/[^s]/.test(m[7]) && (typeof (a) != 'number')) {
				throw ("Expecting number but found " + typeof (a));
			}
			switch (m[7]) {
				case 'b': a = a.toString(2); break;
				case 'c': a = String.fromCharCode(a); break;
				case 'd': a = parseInt(a); break;
				case 'e': a = m[6] ? a.toExponential(m[6]) : a.toExponential(); break;
				case 'f': a = m[6] ? parseFloat(a).toFixed(m[6]) : parseFloat(a); break;
				case 'o': a = a.toString(8); break;
				case 's': a = ((a = String(a)) && m[6] ? a.substring(0, m[6]) : a); break;
				case 'u': a = Math.abs(a); break;
				case 'x': a = a.toString(16); break;
				case 'X': a = a.toString(16).toUpperCase(); break;
			}
			a = (/[def]/.test(m[7]) && m[2] && a > 0 ? '+' + a : a);
			c = m[3] ? m[3] == '0' ? '0' : m[3].charAt(1) : ' ';
			x = m[5] - String(a).length;
			p = m[5] ? str_repeat(c, x) : '';
			o.push(m[4] ? a + p : p + a);
		}
		else {
			throw ("Error!");
		}
		f = f.substring(m[0].length);
	}
	return o.join('');
}

function luhnCalc(number) {
	// Set the string length and parity
	var numberLength = number.length;
	var parity = numberLength % 2;
	// Loop through each digit and do the maths
	var total = 0;

	for (var i = 0; i < numberLength; i++) {
		var digit = number.charCodeAt(i);

		if (digit > 0x39) { // > '9'
			digit -= (0x41 - 10); //  - 'A' + 10
		} else {
			digit -= 0x30;	// '0'
		}
		// Multiply alternate digits by two
		if (i % 2 != parity) {
			digit = digit * 2;
			digit = Math.floor(digit / 10) + (digit % 10);
		}

		// SecOPTIC character
		if ((i % 2 == parity) && (digit > 9)) {
			digit = Math.floor(digit / 10) + (digit % 10);
		}

		// Total up the digits
		total = total + digit;
	}
	// If the total mod 10 equals 0, the number is valid
	return ((10 - (total % 10)) % 10);
}

// Gather the informations, they are necessary to start the xor calculation for specification HHD 1.0.1
function getXorDataV101(startLength, startCode, bde1Length, bde1Code, bde2Length, bde2Code, length, bde1Typ, bde2Typ) {
	var xorData;

	xorData = sprintf("%02X", startLength) + startCode; 			// xor_data: set LS; set Start-Code			

	if (bde1Length > 0) {
		length += 1 + bde1Length;									// LC=LC+LBDE1+(BDE1-length)
		if (bde1Typ == 1) {											// BDE1 is an ASCII value
			bde1Length += 0x10;
		}
		xorData += sprintf("%02X", bde1Length) + bde1Code; 			// xor_data: LS; Start-Code; set LBDE1; set BDE1
	}
	if (bde2Length > 0) {
		length += 1 + bde2Length;									// LC=LC+LBDE2+BDE2-length)	
		if (bde2Typ == 1) {											// BDE2 is an ASCII value
			bde2Length += 0x10;
		}
		xorData += sprintf("%02X", bde2Length) + bde2Code;			// xor_data: LS; Start-Code; LBDE1; BDE1; set LBDE2; set BDE2
	}
	xorData = sprintf("%02X%s", length, xorData);					// xor_data: set LC; LS; Start-Code; LBDE1; BDE1; LBDE2; BDE2
	return xorData;
}

// Gather the informations, they are necessary to start the xor calculation for specification HHD 1.4
function getXorDataV14(startLength, startCode, bde1Length, bde1Code, bde2Length, bde2Code, length, bde3Length, bde3Code, startCodeTyp, bde1Typ, bde2Typ, bde3Typ) {
	var xorData;
	var ls;
	var lsLeftHalfbyte;
	var lsRightHalfbyte;
	var controlByte;
	var lbdex;
	var lbdexLeftHalfbyte;
	var lbdexRightHalfbyte;

	if (startCodeTyp == 1) {																// ASCII values
		ls = getLS(1, 1, startLength);
	} else {																				// Non ASCII values
		ls = getLS(1, 0, startLength);
	}
	lsLeftHalfbyte = getHalfByteDezValue(ls.substr(0, 4));
	lsRightHalfbyte = getHalfByteDezValue(ls.substr(4, 8));
	xorData = sprintf("%X%X", lsLeftHalfbyte, lsRightHalfbyte);								// xor_data: set LS;
	controlByte = 0x1;																		// xor_data: set controlByte (HHD 1.4)
	controlByte = "0" + controlByte;														// xor_data: set controlByte (no other Control Byte)															
	xorData += controlByte;
	xorData += startCode;																	// xor_data: set Start-Code					

	if (bde1Length > 0) {
		length += 1 + bde1Length;															// LC=LC+LBDE1+(BDE1-length)
		if (bde1Typ == 1) {																	// ASCII values		
			lbdex = getLbdex(1, bde1Length);
		} else {																			// Non ASCII values
			lbdex = getLbdex(0, bde1Length);
		}
		lbdexLeftHalfbyte = getHalfByteDezValue(lbdex.substr(0, 4));
		lbdexRightHalfbyte = getHalfByteDezValue(lbdex.substr(4, 8));
		xorData += sprintf("%X%X", lbdexLeftHalfbyte, lbdexRightHalfbyte) + bde1Code;		// xor_data: LS; Start-Code; set LBDE1; set BDE1
	}

	if (bde2Length > 0) {
		length += 1 + bde2Length;															// LC=LC+LBDE2+BDE2-length)	
		if (bde2Typ == 1) {																	// ASCII values		
			lbdex = getLbdex(1, bde2Length);
		} else {																			// Non ASCII values
			lbdex = getLbdex(0, bde2Length);
		}
		lbdexLeftHalfbyte = getHalfByteDezValue(lbdex.substr(0, 4));
		lbdexRightHalfbyte = getHalfByteDezValue(lbdex.substr(4, 8));
		xorData += sprintf("%X%X", lbdexLeftHalfbyte, lbdexRightHalfbyte) + bde2Code;		// xor_data: LS; Start-Code; LBDE1; BDE1; set LBDE2; set BDE2
	}

	if (bde3Length > 0) {
		length += 1 + bde3Length;
		if (bde3Typ == 1) {																	// ASCII values		
			lbdex = getLbdex(1, bde3Length);
		} else {																			// Non ASCII values
			lbdex = getLbdex(0, bde3Length);
		}
		lbdexLeftHalfbyte = getHalfByteDezValue(lbdex.substr(0, 4));
		lbdexRightHalfbyte = getHalfByteDezValue(lbdex.substr(4, 8));
		xorData += sprintf("%X%X", lbdexLeftHalfbyte, lbdexRightHalfbyte) + bde3Code;		// xor_data: LS; Start-Code; LBDE1; BDE1; LBDE2; BDE2; set LBDE3; set BDE3
	}
	xorData = sprintf("%02X%s", length, xorData);											// xor_data: set LC; LS; Start-Code; LBDE1; BDE1; LBDE2; BDE2; LBDE3; BDE3
	return xorData;
}

//Gather the informations, they are necessary to start the xor calculation for specification Stuzza (HHD 1.4)
function getXorDataV14Stuzza(startLength, startCode, bde1Length, bde1Code, bde2Length, bde2Code, length, bde3Length, bde3Code, bde4Length, bde4Code, bde5Length, bde5Code, bde6Length, bde6Code, startCodeTyp, bde1Typ, bde2Typ, bde3Typ, bde4Typ, bde5Typ, bde6Typ) {
	var xorData;
	var ls;
	var lsLeftHalfbyte;
	var lsRightHalfbyte;
	var controlByte;
	var lbdex;
	var lbdexLeftHalfbyte;
	var lbdexRightHalfbyte;

	if (startCodeTyp == 1) {																// ASCII values
		ls = getLS(1, 1, startLength);
	} else {																				// Non ASCII values
		ls = getLS(1, 0, startLength);
	}
	lsLeftHalfbyte = getHalfByteDezValue(ls.substr(0, 4));
	lsRightHalfbyte = getHalfByteDezValue(ls.substr(4, 8));
	xorData = sprintf("%X%X", lsLeftHalfbyte, lsRightHalfbyte);								// xor_data: set LS;
	controlByte = 0x0;																		// xor_data: set controlByte (HHD 1.4)
	controlByte = "8" + controlByte;														// xor_data: set controlByte (other Control Byte)															
	xorData += controlByte;
	secondControlByte = "0" + "9";															// ControlByte for Austria, Version 0001, no other controlByte
	xorData += secondControlByte;
	xorData += startCode;																	// xor_data: set Start-Code					

	if (bde1Length > 0) {
		length += 1 + bde1Length;															// LC=LC+LBDE1+(BDE1-length)
		if (bde1Typ == 1) {																	// ASCII values		
			lbdex = getLbdex(1, bde1Length);
		} else {																			// Non ASCII values
			lbdex = getLbdex(0, bde1Length);
		}
		lbdexLeftHalfbyte = getHalfByteDezValue(lbdex.substr(0, 4));
		lbdexRightHalfbyte = getHalfByteDezValue(lbdex.substr(4, 8));
		xorData += sprintf("%X%X", lbdexLeftHalfbyte, lbdexRightHalfbyte) + bde1Code;		// xor_data: LS; Start-Code; set LBDE1; set BDE1
	}

	if (bde2Length > 0) {
		length += 1 + bde2Length;															// LC=LC+LBDE2+BDE2-length)	
		if (bde2Typ == 1) {																	// ASCII values		
			lbdex = getLbdex(1, bde2Length);
		} else {																			// Non ASCII values
			lbdex = getLbdex(0, bde2Length);
		}
		lbdexLeftHalfbyte = getHalfByteDezValue(lbdex.substr(0, 4));
		lbdexRightHalfbyte = getHalfByteDezValue(lbdex.substr(4, 8));
		xorData += sprintf("%X%X", lbdexLeftHalfbyte, lbdexRightHalfbyte) + bde2Code;		// xor_data: LS; Start-Code; LBDE1; BDE1; set LBDE2; set BDE2
	}

	if (bde3Length > 0) {
		length += 1 + bde3Length;
		if (bde3Typ == 1) {																	// ASCII values		
			lbdex = getLbdex(1, bde3Length);
		} else {																			// Non ASCII values
			lbdex = getLbdex(0, bde3Length);
		}
		lbdexLeftHalfbyte = getHalfByteDezValue(lbdex.substr(0, 4));
		lbdexRightHalfbyte = getHalfByteDezValue(lbdex.substr(4, 8));
		xorData += sprintf("%X%X", lbdexLeftHalfbyte, lbdexRightHalfbyte) + bde3Code;		// xor_data: LS; Start-Code; LBDE1; BDE1; LBDE2; BDE2; set LBDE3; set BDE3
	}

	if (bde4Length > 0) {
		length += 1 + bde4Length;
		if (bde4Typ == 1) {																	// ASCII values		
			lbdex = getLbdex(1, bde4Length);
		} else {																			// Non ASCII values
			lbdex = getLbdex(0, bde4Length);
		}
		lbdexLeftHalfbyte = getHalfByteDezValue(lbdex.substr(0, 4));
		lbdexRightHalfbyte = getHalfByteDezValue(lbdex.substr(4, 8));
		xorData += sprintf("%X%X", lbdexLeftHalfbyte, lbdexRightHalfbyte) + bde4Code;
	}

	if (bde5Length > 0) {
		length += 1 + bde5Length;
		if (bde5Typ == 1) {																	// ASCII values		
			lbdex = getLbdex(1, bde5Length);
		} else {																			// Non ASCII values
			lbdex = getLbdex(0, bde5Length);
		}
		lbdexLeftHalfbyte = getHalfByteDezValue(lbdex.substr(0, 4));
		lbdexRightHalfbyte = getHalfByteDezValue(lbdex.substr(4, 8));
		xorData += sprintf("%X%X", lbdexLeftHalfbyte, lbdexRightHalfbyte) + bde5Code;
	}

	if (bde6Length > 0) {
		length += 1 + bde6Length;
		if (bde6Typ == 1) {																	// ASCII values		
			lbdex = getLbdex(1, bde6Length);
		} else {																			// Non ASCII values
			lbdex = getLbdex(0, bde6Length);
		}
		lbdexLeftHalfbyte = getHalfByteDezValue(lbdex.substr(0, 4));
		lbdexRightHalfbyte = getHalfByteDezValue(lbdex.substr(4, 8));
		xorData += sprintf("%X%X", lbdexLeftHalfbyte, lbdexRightHalfbyte) + bde6Code;
	}

	xorData = sprintf("%02X%s", length, xorData);
	return xorData;
}

//Gather the informations, they are necessary to start the xor calculation for SecOPTIC
function getXorDataSecOPTIC(startLength, startCode, bde1Length, bde1Code, bde2Length, bde2Code, length, startCodeTyp, bde1Typ, bde2Typ) {
	var xorData;
	var ls;
	var lsLeftHalfbyte;
	var lsRightHalfbyte;
	var controlByte;
	var lbdex;
	var lbdexLeftHalfbyte;
	var lbdexRightHalfbyte;

	if (startCodeTyp == 1) {																// ASCII values
		ls = getLS(1, 1, startLength);
	} else {																				// Non ASCII values
		ls = getLS(1, 0, startLength);
	}
	lsLeftHalfbyte = getHalfByteDezValue(ls.substr(0, 4));
	lsRightHalfbyte = getHalfByteDezValue(ls.substr(4, 8));
	xorData = sprintf("%X%X", lsLeftHalfbyte, lsRightHalfbyte);								// xor_data: set LS;
	controlByte = 0x1;																		// xor_data: set controlByte (HHD 1.4)
	controlByte = "0" + controlByte;														// xor_data: set controlByte (no other Control Byte)															
	xorData += controlByte;
	xorData += startCode;																	// xor_data: set Start-Code					

	if (bde1Length > 0) {
		length += 1 + bde1Length;															// LC=LC+LBDE1+(BDE1-length)
		if (bde1Typ == 1) {																	// ASCII values		
			lbdex = getLbdex(1, bde1Length);
		} else {																			// Non ASCII values
			lbdex = getLbdex(0, bde1Length);
		}
		lbdexLeftHalfbyte = getHalfByteDezValue(lbdex.substr(0, 4));
		lbdexRightHalfbyte = getHalfByteDezValue(lbdex.substr(4, 8));
		xorData += sprintf("%X%X", lbdexLeftHalfbyte, lbdexRightHalfbyte) + bde1Code;		// xor_data: LS; Start-Code; set LBDE1; set BDE1
	}

	if (bde2Length > 0) {
		length += 1 + bde2Length;															// LC=LC+LBDE2+BDE2-length)	
		if (bde2Typ == 1) {																	// ASCII values		
			lbdex = getLbdex(1, bde2Length);
		} else {																			// Non ASCII values
			lbdex = getLbdex(0, bde2Length);
		}
		lbdexLeftHalfbyte = getHalfByteDezValue(lbdex.substr(0, 4));
		lbdexRightHalfbyte = getHalfByteDezValue(lbdex.substr(4, 8));
		xorData += sprintf("%X%X", lbdexLeftHalfbyte, lbdexRightHalfbyte) + bde2Code;		// xor_data: LS; Start-Code; LBDE1; BDE1; set LBDE2; set BDE2
	}

	xorData = sprintf("%02X%s", length, xorData);											// xor_data: set LC; LS; Start-Code; LBDE1; BDE1; LBDE2; BDE2; LBDE3; BDE3
	return xorData;

}

function xorCalc(number) {
	var numberLength = number.length;
	// Loop through each digit and do the maths
	var total = 0;
	for (var i = 0; i < numberLength; i++) {
		var digit = number.charCodeAt(i);
		if (digit > 0x39) { // > '9'
			digit -= (0x41 - 10); //  - 'A' + 10
		} else {
			digit -= 0x30;	// '0'
		}
		// Total up the digits
		total ^= digit;
	}
	return (total);
}

// Returns the constructed LS byte
function getLS(controlByte, startCodeTyp, startLength) {
	var ls = sprintf("%b", startLength);

	for (var i = ls.length; i < 6; i++) {
		ls = "0" + ls;
	}
	ls = startCodeTyp + ls;
	ls = controlByte + ls;
	return ls;
}

function str_repeat(i, m) {
	for (var o = []; m > 0; o[--m] = i);
	return (o.join(''));
}

// Returns the decimal value from a half byte
function getHalfByteDezValue(halfByte) {
	var dezValue;

	switch (halfByte) {
		case "0000":
			dezValue = 0;
			break;
		case "0001":
			dezValue = 1;
			break;
		case "0010":
			dezValue = 2;
			break;
		case "0011":
			dezValue = 3;
			break;
		case "0100":
			dezValue = 4;
			break;
		case "0101":
			dezValue = 5;
			break;
		case "0110":
			dezValue = 6;
			break;
		case "0111":
			dezValue = 7;
			break;
		case "1000":
			dezValue = 8;
			break;
		case "1001":
			dezValue = 9;
			break;
		case "1010":
			dezValue = 10;
			break;
		case "1011":
			dezValue = 11;
			break;
		case "1100":
			dezValue = 12;
			break;
		case "1101":
			dezValue = 13;
			break;
		case "1110":
			dezValue = 14;
			break;
		case "1111":
			dezValue = 15;
			break;
	}
	return dezValue;
}

// Returns the constructed LBDEX byte
function getLbdex(bdeTyp, bdeLength) {
	var ldbex = sprintf("%b", bdeLength);

	for (var i = ldbex.length; i < 6; i++) {
		ldbex = "0" + ldbex;
	}
	ldbex = bdeTyp + ldbex;
	ldbex = "0" + ldbex;
	return ldbex;
}
