# WfgAdvancedChartComponent

## Usage

As of the `JavaScript` version it is necessary to instantiate an object of the `WfgAdvancedChartComponent` class

Teletrader source It is necessary to add the teletrader resource located in the path "`[assetbase]/wfgassets/teletrader/TTChart.js`"

If you want to implement the option to add the lightstreamer functionality, you need to add the "lightstreamer-web-client" resource located in the path "`[assetbase]/wfgassets/lightstreamer-web-client/lightstreamer.min.js`"

The class requires two parameters:
- Css selector or Element ref (HTMLElement)
- options: an object with options. Avaliable options:
    - **height**: string (mandatory) altura del componente. Valores aceptados: `'100%'`, `'100px'`, `'100vh'`.
    - **locale**: string (mandatory) the locale ISO codes ISO-639 + '-' + ISO-3166 (for example: "de-CH"). Available values: `'de-CH'` | `'en-CH'` | `'en-EN'` | `'fr-FR'` | `'it-CH'` | `'it-IT'`
    - **serviceurl**: string (mandatory) base url of the service
    - **assetbase**: string (mandatory). Indicates the location of the resources. Path to folder `wfgassets`.
    - **instrumentid**: string (optional) instrumentid the instrument id you want to get the information in the header (default: `35395118-6921-814` for `Unicredit Rg`)
    - **period**: string (optional) the default period. If not indicated, the default value of the widget is taken. Available values: `INTRADAY`, `DAILY`, `WEEKLY`, `MONTHLY`, `YEARSLY`, and `TICK`
    - **timeframe**: number (optional) default frequency. If not indicated, the default value of the widget is taken. Available values: `1`, `5`, `10`, `15`, `30`, and `60`
    - **token**: string (optional) token to data service.
    - **lstoken**: string (optional) token to LightStreamer service, if no token is provided LightStreamer service does not start the session.
    - **removemenuoptions**: string (opcional)  Do not show toolbar options on the chart. Comma Separated Values. For example (`print,volume`). The toolbar options are: `dateRange`, `periods`, `indicators`, `chartType`, `volume`, `history`, `grid`, `relativeScale`, `logarithmicScale`, `dataInfo`, `last`, `fullscreen`, `crosshair`, `zoomOut`, `zoomIn`, `reload`, `print`. Default: `print,grid,relativeScale,logarithmicScale`
    - **simpleindicators**: boolean (optional) `true` or `false`. Show the simple indicators or the advanced ones
    - **maxindicators**: number (optional). Indicates the number of indicators that can be added to the chart
    - **showsearch**: boolean (optional) `true` by default. Indicates whether or not to display the search component for comparisons (`true` by default).
    - **showvolume**: boolean (optional). `false` by default. Indicates whether or not to display the chart volume serie (`false` by default).
    - **showhistory**: boolean (optional). `false` by default. Indicate whether or not to display the chart history section (`false` by default).
    - **onlyverticaltoolbar**: boolean (optional). `true` by default. Indicate whether or not to display the pencil toolbar (`true` by default).
    - **onlyicons**: boolean (optional). `true` by default. Indicate whether or not to display the icons by default (`true` by default).
    - **ttChartDefaultOpts**: Object (optional). Default baha chart options (`{'main.chartType': 'mountain'}` by default).

### Methods

You can chenge, at runtime, some of the parameters indicated when constructing the `WfgAdvancedChartComponent` object:

- `setLocale(locale)`: where `"locale"` is a string. Change the locale value and run the translations. `"locale"` parameter accepts the same values as the parameter listed above.
- `setInstrumentid(instrumentid)`: where `"instrumentid"` is an instrument identifier (Listing id or ID NOTATION mode). Change de main instrument id.
- `setServiceurl(serviceurl)`: where `"serviceurl"` is the base service url (string).
- `setShowsearch(showsearch)`: show or hide search input. Where `"showsearch"` is `true` or `false`.
- `setToken(token)`: modify the `token`.
- `setLsToken(lstoken)`: modify the `token` used for `lightstreamer`. If indicated, the connection is made to the socket.
- `setSimpleindicators(simpleindicators)`: indicates whether the `indicators` should be shown in a simple format or the complete list of `indicators`.
- `setPeriod(period)`: modifies the value of the period field. Possible values are: `INTRADAY`, `DAILY`, `WEEKLY`, `MONTHLY`, `YEARSLY`, and `TICK`.
- `setTimeframe(timeframe)`: modifies the value of the timeframe field. Possible values are: `1`, `5`, `10`, `15`, `30`, and `60`.
- `setRemovemenuoptions(removemenuoptions)`: modifies the delete values in the menu options. Values must be indicated separated by commas. Available values are: `dateRange`, `periods`, `indicators`, `chartType`, `volume`, `history`, `grid`, `relativeScale`, `logarithmicScale`, `dataInfo`, `last`, `fullscreen`, `crosshair`, `zoomOut`, `zoomIn`, `reload`, `print`.
- `setShowVolume(showvolume)`: indicates whether or not the volume of the graph is displayed.
- `setShowHistory(showhistory)`: indicates whether or not the graph history is shown.
- `setOnlyverticaltoolbar(onlyverticaltoolbar)`: indicates whether or not the pencil tool is shown.
- `setOnlyicons(onlyicons)`: indicates whether or not the icons are shown.


### Example

The layout page: 
```HTML
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Pure Vanilla JS TTChart</title>
    <style>
        html, body{
          margin: 0; padding: 0;
        }
    </style>
</head>
<body>
    <div id="main-chart"></div>
    <script src="./wfgassets/teletrader/TTChart.js"></script>
    <script src="./wfgassets/lightstreamer-web-client/lightstreamer.min.js"></script>
    <script src="./wfgassets/purejs-ttchart-min.js"></script>
    <script src="./js/main.js"></script>
</body>
</html>
```

```JS
// ################
// # ./js/main.js #
// ################
// Unicredit Rg
const instrumentid = '35395118-6921-814';
const access_token = [valid token];
const baseOptions = {
    removemenuoptions: 'dateRange,periods,indicators,chartType,volume,history,grid,relativeScale,logarithmicScale,dataInfo,last,fullscreen,crosshair,zoomOut,zoomIn,reload,print',
    height: '65vh',
    locale: 'it-IT',
    serviceurl: 'https://unicredit-middle-gcp-uat.webfg.com/v1/Unicredit',
};
const elem = document.querySelector('#main-chart');
const advancedChartComponent = new WfgAdvancedChartComponent('#main-chart' /* or elem */, {
    ...baseOptions,
    token: access_token,
    lstoken: access_token,
    instrumentid: instrumentid
});
// change language
advancedChartComponent.setLocale('en-EN');
// change the main instrument
advancedChartComponent.setInstrumentid('30827865-6921-814');
// hide search input
advancedChartComponent.setShowsearch(false);
```

## TTChart custom translations:

### EN

```MD
# teletrader/locales/en.txt
wfg_range_day:Day
wfg_range_week:Week
wfg_range_month:Month
wfg_range_ytd:YTD
```
### DE

```MD
# teletrader/locales/de.txt
wfg_range_day:Tag
wfg_range_week:Woche
wfg_range_month:Monat
wfg_range_ytd:YTD
```
### FR

```MD
# teletrader/locales/fr.txt
wfg_range_day:Jour
wfg_range_week:Sema.
wfg_range_month:Mois
wfg_range_ytd:YTD
```
### IT

```MD
# teletrader/locales/it.txt
wfg_range_day:Gior.
wfg_range_week:Sett.
wfg_range_month:Mese
wfg_range_ytd:YTD
```
